#include "adc.h"
#include "delay.h"
#include "stm32f30x.h"

static adc_data_cb adc_data_callback;

enum { ADC_HALF_BUFFER = 512 };
unsigned short ADCbuf[2*ADC_HALF_BUFFER];


static void setup_adc_gpio(void)
{
	GPIO_InitTypeDef       GPIO_InitStructure;
	/* Enable GPIOA Periph clock */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	/* Configure ADC1/2 channels as analog input */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_4;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

static void setup_adc_dma(void)
{
	DMA_InitTypeDef        DMA_InitStructure;
	/* Enable DMA1 clock */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

	/* DMA1 Channel1 Init */
	DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&ADC1->DR;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;	
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)ADCbuf;
	DMA_InitStructure.DMA_BufferSize = sizeof(ADCbuf)/sizeof(ADCbuf[0]);
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord; // DMA_PeripheralDataSize_Byte;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord; // DMA_MemoryDataSize_Byte;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	DMA_InitStructure.DMA_Priority = DMA_Priority_Medium;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

	DMA_Init(DMA1_Channel1, &DMA_InitStructure);

	/* DMA interrupts */
	DMA_ITConfig(DMA1_Channel1, DMA_IT_TC | DMA_IT_HT, ENABLE);

	/* enable DMA IRQ */
	NVIC_EnableIRQ(DMA1_Channel1_IRQn);
} 

static void calibrate_adc(void)
{
	__IO uint16_t calibration_value_1 = 0;	///< __IO = volatile	

  /* ADC Calibration procedure */
  ADC_VoltageRegulatorCmd(ADC1, ENABLE);
  
  /* Insert delay equal to 10 us */
  SystickDelay(10);
  
  ADC_SelectCalibrationMode(ADC1, ADC_CalibrationMode_Single);
  ADC_StartCalibration(ADC1);

  while(ADC_GetCalibrationStatus(ADC1) != RESET );
  calibration_value_1 = ADC_GetCalibrationValue(ADC1);
}

void setup_adc(adc_data_cb on_adc_data)
{
	ADC_InitTypeDef        ADC_InitStructure;
		
	adc_data_callback = on_adc_data;

	/* Configure the ADC clock */
	// Tconv = Sampling time + 8.5 (for 8-bit resolution) ADC clock cycles 
	RCC_ADCCLKConfig(RCC_ADC12PLLCLK_Div6);  
	/* Enable ADC1 clock */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_ADC12, ENABLE);
		
	setup_adc_gpio();
	setup_adc_dma();
	
  /* Setup SysTick Timer for 1 sec interrupts  */
  if (SysTick_Config(SystemCoreClock / 1000000))
  { 
    Buzzer_On();
		/* Capture error */ 
    while (1)
    {}
  }	
  
	calibrate_adc();

	/* ADC setup */  
	ADC_StructInit(&ADC_InitStructure);
	ADC_InitStructure.ADC_ContinuousConvMode = ADC_ContinuousConvMode_Enable;
	ADC_InitStructure.ADC_Resolution = ADC_Resolution_8b; 
	ADC_InitStructure.ADC_ExternalTrigConvEvent = ADC_ExternalTrigConvEvent_0;         
	ADC_InitStructure.ADC_ExternalTrigEventEdge = ADC_ExternalTrigEventEdge_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_OverrunMode = ADC_OverrunMode_Disable;   
	ADC_InitStructure.ADC_AutoInjMode = ADC_AutoInjec_Disable;  
	ADC_InitStructure.ADC_NbrOfRegChannel = 1;
	ADC_Init(ADC1, &ADC_InitStructure);

  ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_7Cycles5);

	
	/* Configures the ADC DMA */
	ADC_DMAConfig(ADC1, ADC_DMAMode_Circular);
		
	/* Enable the ADC DMA */
	ADC_DMACmd(ADC1, ENABLE);

  /* Enable ADC1 */
  ADC_Cmd(ADC1, ENABLE);

	/* wait for ADC1 ADRDY */
	while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_RDY));
	
	/* Enable the DMA channel */
	DMA_Cmd(DMA1_Channel1, ENABLE);

	/* Start ADC1 Software Conversion */ 
	ADC_StartConversion(ADC1);
}

/* Handles DMA1_Channel1 (ADC Buffer) interrupt request */
void DMA1_Channel1_IRQHandler(void) __attribute__ ((section (".ccmram")));
void DMA1_Channel1_IRQHandler(void)
{
	unsigned short *ptr;
	
	if(DMA1->ISR & DMA1_IT_HT1)
	{
		/* Half transfer - 1st half buffer has data */
		ptr = &ADCbuf[0];
	}
	else
	{
		/* Transfer complete - 2nd half buffer has data */
		ptr = &ADCbuf[ADC_HALF_BUFFER];
	}

	/* process the buffer */
	adc_data_callback(ptr, ADC_HALF_BUFFER);		
	
	/* Clear DMA1_Channel1 interrupt */
	DMA1->IFCR = DMA_IFCR_CGIF1;
}
